/*
 * This is an example client that attaches to an account factory
 * and uses it to create accounts and manipulate the
 */

#include <stdio.h>

/* Include the ORBit orb header */
#include <orb/orbit.h>

/* include the header file generated by orbit-idl. This contains the
 * declarations of the client side functions and a few other things.  */
#include "factory.h"


int main (int argc, char *argv[]) {

  factory fac;		      /* The server-side factory object */
  Account acc1, acc2;	      /* Our victim Accounts */

  CORBA_ORB orb=NULL;         /* This is our ORB */
  char *ior;		      /* an IOR is one way a client can locate
				 a server */
  CORBA_Environment ev;       /* CORBA exception information */

  CORBA_exception_init(&ev);

  /* Initialize the orb. The initialization routine checks the command
   * line parameters for directives that affect the orb */
  orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);

  if (argc != 2) {
    fprintf(stderr, "Must supply an IOR!\n");
    exit(1);
  }

  ior = argv[1];

  /* Translate the IOR into a pointer to our object */
  fac = CORBA_ORB_string_to_object(orb, ior, &ev);

  /* Use the factory to create a couple of accounts */
  acc1 = factory_newAccount(fac,0,&ev);
  acc2 = factory_newAccount(fac,40,&ev);

  /* Do stuff with the accounts */
  Account_deposit(acc1, (CORBA_unsigned_long) 12, &ev);
  Account_deposit(acc2, (CORBA_unsigned_long) 12, &ev);

  val = Account__get_balance(acc1, &ev);
  printf ("Balance of account 1 is %d.\n",val);
  val = Account__get_balance(acc2, &ev);
  printf ("Balance of account 2 is %d.\n",val);

  Account_withdraw(acc1, (CORBA_unsigned_long) 5, &ev);
  Account_withdraw(acc2, (CORBA_unsigned_long) 5, &ev);

  val = Account__get_balance(acc1, &ev);
  printf ("Balance of account 1 is %d.\n",val);
  val = Account__get_balance(acc2, &ev);
  printf ("Balance of account 2 is %d.\n",val);
 

  /* We're done running, clean up */
  CORBA_Object_release(fac, &ev);
  CORBA_Object_release(acc1, &ev);
  CORBA_Object_release(acc2, &ev);

  CORBA_ORB_shutdown(orb, CORBA_FALSE, &ev);


}


